"""Storage Backends for Empathy Framework Memory

Provides pluggable storage backends for pattern persistence:
- File-based (default): Simple JSON file storage
- SQLite (planned): Local database for better querying
- S3/Cloud (planned): Distributed storage for teams

Copyright 2025 Smart AI Memory, LLC
Licensed under Fair Source 0.9
"""

# Storage backends will be added here as they're implemented
# Current implementation uses file-based storage in long_term.py

__all__ = []
